; 时钟的例子程序

option casemap:none

.data
; 常量定义
CS_VREDRAW          equ 1
CS_HREDRAW          equ 2
CW_USEDEFAULT       equ 80000000h
WS_OVERLAPPEDWINDOW equ 0CF0000h
WM_TIMER            equ 0113h
WM_DESTROY          equ 2
WM_PAINT            equ 0Fh
ID_TIMER            equ 1
DT_CENTER equ 1
DT_VCENTER equ 4
DT_SINGLELINE equ 20h
COLOR_WINDOW    equ 5

; 结构体定义
WNDCLASSEX struct
    cbSize          dd ?
    style           dd ?
    lpfnWndProc     dq ?
    cbClsExtra      dd ?
    cbWndExtra      dd ?
    hInstance       dq ?
    hIcon           dq ?
    hCursor         dq ?
    hbrBackground   dq ?
    lpszMenuName    dq ?
    lpszClassName   dq ?
    hIconSm         dq ?
WNDCLASSEX ends

PAINTSTRUCT struct
    hdc         dq ?
    fErase      dd ?
    rcPaint     db 16 dup(?)
    fRestore    dd ?
    fIncUpdate  dd ?
    rgbReserved db 32 dup(?)
PAINTSTRUCT ends

SYSTEMTIME struct
    wYear           dw ?
    wMonth          dw ?
    wDayOfWeek      dw ?
    wDay            dw ?
    wHour           dw ?
    wMinute         dw ?
    wSecond         dw ?
    wMilliseconds   dw ?
SYSTEMTIME ends

; 数据定义
className     dw 'M','y','W','i','n','d','o','w',0
windowName    dw 'R','e','a','l','t','i','m','e',' ','C','l','o','c','k',0
timeBuffer    db '00:00:00',0
hInstance     dq 0
hWnd          dq 0
sysTime       SYSTEMTIME <>
wc            WNDCLASSEX <>
msg           db 48 dup(0) ; MSG结构体
ps            PAINTSTRUCT <>

.code
extern GetModuleHandleA: proc
extern RegisterClassExW: proc
extern CreateWindowExW: proc
extern ShowWindow: proc
extern UpdateWindow: proc
extern GetMessageW: proc
extern TranslateMessage: proc
extern DispatchMessageW: proc
extern DefWindowProcW: proc
extern SetTimer: proc
extern KillTimer: proc
extern GetLocalTime: proc
extern BeginPaint: proc
extern EndPaint: proc
extern DrawTextA: proc
extern PostQuitMessage: proc
extern ExitProcess: proc
extern InvalidateRect: proc

WindowProc proc
    ; rcx = hwnd, rdx = uMsg, r8 = wParam, r9 = lParam
    sub rsp, 28h ; Shadow space + align

    cmp edx, WM_TIMER
    je wm_timer
    cmp edx, WM_DESTROY
    je wm_destroy
    cmp edx, WM_PAINT
    je wm_paint

default:
    ; 默认消息处理
    call DefWindowProcW
    jmp finish

wm_timer:
    ; 更新时间并重绘
    lea rcx, sysTime
    call GetLocalTime

    ; 转换小时
    mov ax, [sysTime.wHour]
    xor dx, dx
    mov cx, 10
    div cx
    add al, '0'
    mov [timeBuffer], al
    add dl, '0'
    mov [timeBuffer+1], dl

    ; 转换分钟
    mov ax, [sysTime.wMinute]
    xor dx, dx
    mov cx, 10
    div cx
    add al, '0'
    mov [timeBuffer+3], al
    add dl, '0'
    mov [timeBuffer+4], dl

    ; 转换秒
    mov ax, [sysTime.wSecond]
    xor dx, dx
    mov cx, 10
    div cx
    add al, '0'
    mov [timeBuffer+6], al
    add dl, '0'
    mov [timeBuffer+7], dl

    ; 请求重绘
    mov rcx, [hWnd]
    xor rdx, rdx
    xor r8, r8
    call InvalidateRect
    jmp finish

wm_paint:
    ; 绘制文本
    mov rcx, [hWnd]
    lea rdx, ps
    call BeginPaint

    ; 设置绘制参数
    mov rcx, rax ; hdc
    lea rdx, timeBuffer
    mov r8, -1   ; 字符串长度自动计算
    lea r9, [ps.rcPaint]
    mov dword ptr [rsp+20h], DT_CENTER or DT_VCENTER or DT_SINGLELINE
    call DrawTextA

    mov rcx, [hWnd]
    lea rdx, ps
    call EndPaint
    jmp finish

wm_destroy:
    ; 销毁定时器并退出
    mov rcx, [hWnd]
    mov rdx, ID_TIMER
    call KillTimer

    xor ecx, ecx
    call PostQuitMessage

finish:
    add rsp, 28h
    ret
WindowProc endp

main proc
    sub rsp, 28h ; 对齐栈

    ; 获取模块句柄
    xor rcx, rcx
    call GetModuleHandleA
    mov rcx, rax

    ; 初始化窗口类
    mov [wc.cbSize], SIZEOF WNDCLASSEX
    mov [wc.style], CS_HREDRAW or CS_VREDRAW
    lea rax, WindowProc
    mov [wc.lpfnWndProc], rax
    mov wc.cbClsExtra, 0
    mov wc.cbWndExtra, 0
    mov [wc.hInstance], rcx
    lea rax, [className]
    mov [wc.lpszClassName], rax
    mov wc.hbrBackground, COLOR_WINDOW + 1
    mov wc.lpszMenuName, 0
    mov wc.hIcon, 0
    mov wc.hIconSm, 0

    ; 注册窗口类
    lea rcx, wc
    call RegisterClassExW
    test ax, ax
    jz exit

    ; 创建窗口
    xor rcx, rcx
    lea rdx, className
    lea r8, windowName
    mov r9d, WS_OVERLAPPEDWINDOW
    mov dword ptr [rsp+20h], CW_USEDEFAULT
    mov dword ptr [rsp+28h], CW_USEDEFAULT
    mov qword ptr [rsp+30h], 300 ; 宽度
    mov qword ptr [rsp+38h], 200 ; 高度
    mov qword ptr [rsp+40h], 0   ; 父窗口
    mov qword ptr [rsp+48h], 0   ; 菜单
    lea rax, [wc.hInstance]
    mov qword ptr [rsp+50h],  rax
    mov qword ptr [rsp+58h], 0
    call CreateWindowExW
    mov [hWnd], rax

    ; 设置定时器
    mov rcx, rax
    mov rdx, ID_TIMER
    mov r8, 1000 ; 间隔1秒
    xor r9, r9
    call SetTimer

    ; 显示窗口
    mov rcx, [hWnd]
    mov rdx, 1 ; SW_SHOWNORMAL
    call ShowWindow

    ; 消息循环
msg_loop:
    lea rcx, msg
    xor rdx, rdx
    xor r8, r8
    xor r9, r9
    call GetMessageW
    test eax, eax
    jz exit

    lea rcx, msg
    call TranslateMessage

    lea rcx, msg
    call DispatchMessageW

    jmp msg_loop

exit:
    xor ecx, ecx
    call ExitProcess
main endp

end